<?php

namespace App\Livewire\Admin\Product;

use App\Models\Product;
use App\Models\ProductDetail;
use App\Models\Storage;
use App\Models\SubCategory;
use App\Services\FileService;
use App\Services\ProductServices;
use Livewire\Component;
use Livewire\Features\SupportFileUploads\TemporaryUploadedFile;
use Livewire\WithFileUploads;
use WireUi\Traits\WireUiActions;

class Edit extends Component
{
    use WireUiActions, WithFileUploads;

    public $name;
    public $description;
    public $advantage;
    public $category_id;


    public array $new_packaging_details = [
        [
            'tittle' => null,
            'value' => null,
        ]
    ];
    public array $packaging_details = [
        [
            'tittle' => null,
            'value' => null,
        ]
    ];

    public $main_photo = null;

    public array $photos = [];

    public array $existingImages = [];

    public array $removedImages = [];

    public $newPhoto;

    public Product $product;

    public function mount()
    {
        $this->name = $this->product->name;
        $this->description = $this->product->description;
        // Fetch product details and convert them to an array
        $this->category_id = $this->product->category_id;
        $this->photos = $this->product->otherImages->pluck('file_path')->toArray();
        $this->main_photo = $this->product->mainImage->file_path ?? null;
        $detail_array = $this->product->packagingDetail ? $this->product->packagingDetail->toArray() : [];
        if (isset($detail_array['characteristics']) && is_array($detail_array['characteristics'])) {
            foreach ($detail_array['characteristics'] as $key => $value) {
                $this->packaging_details[] = [
                    'tittle' => $key,
                    'value' => $value,
                ];
            }
        } else {
            // Handle the case where 'characteristics' doesn't exist or is not an array
            $this->packaging_details = []; // Or handle it differently if needed
        }
        $this->advantage = $this->product->packagingDetail->advantages ?? null;
    }

    public function update(ProductServices $productServices, FileService $fileService)
    {

        $this->validate([
            'name' => 'required|string|max:255',
            'description' => 'required',
        ]);

        // try {
        $data = [
            'name' => $this->name,
            'description' => $this->description,
            'category_id' => $this->category_id,
        ];

        $productServices->update($this->product, $data);
        ProductDetail::where('product_id', $this->product->id)->delete();
        $this->packaging_details = array_filter($this->packaging_details, function ($entry) {
            return !empty($entry['tittle']) && !is_null($entry['value']);
        });
        ProductDetail::create([
            'product_id' => $this->product->id,
            'advantages' => $this->advantage,
            'characteristics' => json_encode(array_column($this->packaging_details, 'value', 'tittle')),
        ]);


        if ($this->main_photo && $this->isTemporaryUploadedFile($this->main_photo)) {
            // Remove file from storage
            if ($this->product->mainImage != null && $this->product->mainImage->file_path) {
                $fileService->deleteFile(
                    $this->product->mainImage->file_path,
                    'products',
                );
            }

            // Add New image
            $upload = $fileService->uploadFile(
                $this->main_photo,
                'products',
            );
            Storage::updateOrCreate(
                ['table_id' => $this->product->id, 'table_type' => 'product_main_image'],
                ['file_path' => $upload['path']]
            );
        }

        if ($this->photos) {
            foreach ($this->photos as $value) {
                if ($this->isTemporaryUploadedFile($value)) {
                    $uploaded = $fileService->uploadFile(
                        $value,
                        'products',
                    );
                    Storage::create([
                        'file_path' => $uploaded['path'],
                        'table_id' => $this->product->id,
                        'table_type' => 'product_other_image',
                    ]);
                }
            }
        }

        if (count($this->removedImages) > 0) {
            foreach ($this->removedImages as $discartImages) {
                // Remove file from storage
                $fileService->deleteFile(
                    $discartImages,
                    'products',
                );
                // Remove the storage database values
                Storage::where([
                    'table_id' => $this->product->id,
                    'table_type' => 'product_other_image',
                    'file_path' => $discartImages,
                ])->delete();
            }
        }

        $this->notification()->success('Product updated successfully.');

        return redirect()->to(route('products.index'));
        // } catch (\Illuminate\Validation\ValidationException $e) {
        //     $this->notification()->error($e->getMessage());
        // } catch (\Exception $e) {
        //     $this->notification()->error('Error: An unexpected error occurred.');
        // }
    }

    public function addMorePackagingDetail(): void
    {
        $this->packaging_details[] = [
            'tittle' => null,
            'value' => null,
        ];
    }

    public function removePackagingDetail($index): void
    {
        unset($this->packaging_details[$index]);
        $this->packaging_details = array_values($this->packaging_details); // Re-index the array
    }

    public function updatedNewPhoto(): void
    {
        $this->photos[] = $this->newPhoto;
    }

    public function removePhoto($index): void
    {
        if (! $this->isTemporaryUploadedFile($this->photos[$index])) {
            $this->removedImages[] = $this->photos[$index];
        }
        array_splice($this->photos, $index, 1);
    }

    public function isTemporaryUploadedFile($photo): bool
    {
        if ($photo instanceof TemporaryUploadedFile) {
            return true;
        }

        return false;
    }

    public static function cleanUrlV2($str): string
    {
        $clean = preg_replace("/[^a-zA-Z0-9\/_|+ -]/", '-', $str);
        $clean = strtolower(trim($clean, '-'));
        $clean = preg_replace("/[\/_|+ -]+/", '-', $clean);

        return $clean;
    }

    public function render()
    {
        $categories = SubCategory::where('status', true)->whereHas('category')->with('category')->get();
        return view('livewire.admin.product.edit', ['categories' => $categories])->layout('components.layouts.admin');
    }
}
