<?php

namespace App\Services;

use App\Models\CampaignKeyword;
use App\Models\CampaignPerformanceMetric;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Http;

class AmazonAdsService
{
    protected $client;
    protected $baseUrl = 'https://advertising-api-eu.amazon.com';

    public function __construct()
    {
        $this->client = new Client([
            'verify' => false,// Bypass SSL verification
        ]);
    }

    /**
     * Generate Amazon OAuth Authorization URL
     */
    public function getAuthorizationUrl()
    {
        return 'https://eu.account.amazon.com/ap/oa' . http_build_query([
            'client_id' => env('AMAZON_CLIENT_ID'),
            'scope' => 'advertising::campaign_management',
            'response_type' => 'code',
            'redirect_uri' => env('AMAZON_REDIRECT_URI'),
        ]);
    }

    /**
     * Exchange Authorization Code for Refresh Token
     */
    public function getRefreshToken($authorizationCode)
    {
        $response = $this->client->post('https://api.amazon.co.uk/auth/o2/token', [
            'form_params' => [
                'grant_type' => 'authorization_code',
                'code' => $authorizationCode,
                'client_id' => env('AMAZON_CLIENT_ID'),
                'client_secret' => env('AMAZON_CLIENT_SECRET'),
                'redirect_uri' => env('AMAZON_REDIRECT_URI'),
            ],
        ]);

        $data = json_decode($response->getBody(), true);
        if (isset($data['refresh_token'])) {
            return $data['refresh_token'];
        }

        Log::error('Failed to get refresh token', $data);
        return null;
    }

    /**
     * Get Profile ID
     */
    public function getProfileId($accessToken)
    {
        $response = $this->client->get($this->baseUrl . '/v2/profiles', [
            'headers' => [
                'Authorization' => 'Bearer ' . $accessToken,
                'Amazon-Advertising-API-ClientId' => env('AMAZON_CLIENT_ID'),
            ],
        ]);

        $profiles = json_decode($response->getBody(), true);

        if (!empty($profiles) && isset($profiles[0]['profileId'])) {
            foreach($profiles as $profile){
                if($profile['countryCode']=='UK'){
                    return $profile['profileId'];
                }
            }
            return $profiles[0]['profileId'];
        }
        Log::error('Failed to get Profile ID', $profiles);
        return null;
    }

    /**
     * Get Access Token Using Refresh Token
     */
    public function getAccessToken($token=null)
    {
        $refreshToken = $token ?? env('AMAZON_REFRESH_TOKEN');
        if (!$refreshToken) {
            Log::error('Refresh token is missing.');
            return null;
        }
        $response = $this->client->post('https://api.amazon.co.uk/auth/o2/token', [
            'form_params' => [
                'grant_type' => 'refresh_token',
                'refresh_token' => $refreshToken,
                'client_id' => env('AMAZON_CLIENT_ID'),
                'client_secret' => env('AMAZON_CLIENT_SECRET'),
            ],
        ]);

        $data = json_decode($response->getBody(), true);
        return $data['access_token'] ?? null;
    }

    /**
     * Fetch Amazon Ad Campaigns
     */
    public function getCampaigns($accessToken,$profile_Id=null)
    {
        $accessToken = $accessToken ?? $this->getAccessToken();
        if (!$accessToken) {
            return null;
        }

        $profileId = $profile_Id ?? env('AMAZON_PROFILE_ID');
        if (!$profileId) {
            // dd(1);
            $profileId = $this->getProfileId($accessToken);
        }
        // dd($profileId);

        $response = $this->client->post($this->baseUrl . '/sp/campaigns/list', [
            'headers' => [
                'Authorization' => 'Bearer ' . $accessToken,
                'Amazon-Advertising-API-ClientId' => env('AMAZON_CLIENT_ID'),
                'Amazon-Advertising-API-Scope' => $profileId,
                // 'Amazon-Advertising-API-Scope' =>"1723105052499887",
                'Content-Type' => 'application/vnd.spCampaign.v3+json',
                'Accept'=>'application/vnd.spCampaign.v3+json'
            ],
            'json' => (object) [] ,// Add this empty JSON object this json is for filetr purpose
        ]);
        return json_decode($response->getBody()->getContents(), true);
    }


    /**
     * Get Selling Partner ID using the access token
     * 
     * @param string $accessToken
     * @return string|null
     */
    public function getSellingPartnerId($accessToken)
    {
        // Define the API endpoint for getting marketplace participation (which includes sellingPartnerId)
        $apiUrl = 'https://sellingpartnerapi-na.amazon.com/sellers/v1/marketplaceParticipations';

        // try {
            // Make the API request to get the selling partner ID
            $response = $this->client->request('GET', $apiUrl, [
                'headers' => [
                    'Authorization' => 'Bearer ' . $accessToken,
                    'Content-Type' => 'application/json',
                ]
            ]);

            // Decode the JSON response
            $data = json_decode($response->getBody()->getContents(), true);

            // Check if the response contains marketplaceParticipations and sellingPartnerId
            if (isset($data['marketplaceParticipations'][0]['sellerId'])) {
                return $data['marketplaceParticipations'][0]['sellerId']; // This is the selling partner ID
            }

            // If selling partner ID is not found, return null
            return null;

        // } catch (\Exception $e) {
        //     Log::error('Failed to fetch selling partner ID: ' . $e->getMessage());
        //     return null;
        // }
    }

    public function getCampaignKeywords($accessToken,$profile_Id,$campaignId)
    {
        // Get the access token
        $profileId = $profile_Id ?? env('AMAZON_PROFILE_ID');
        // Set the API endpoint
        $url = $this->baseUrl."/sp/keywords/list";
        // Request Body
            

        $response = $this->client->post($url, [
            'headers' => [
                'Authorization' => 'Bearer ' . $accessToken,
                'Amazon-Advertising-API-ClientId' => env('AMAZON_CLIENT_ID'),
                'Amazon-Advertising-API-Scope' => $profileId, // Profile ID for your ads account
                      'Content-Type' => 'application/vnd.spKeyword.v3+json',  // Correct Content-Type
                        'Accept' => 'application/vnd.spKeyword.v3+json'   ,
            ],      // Correct Accept header
            'json' => [ // Add the body here
                "campaignIdFilter" => [
                    "include" => [$campaignId]
                ],
                "stateFilter" => [
                    "include" => ["ENABLED", "PAUSED", "ARCHIVED"]
                ],
                "maxResults" => 100,
                "includeExtendedDataFields" => true,
                "locale" => "en_US"
            ]
        ]);

        $data = json_decode($response->getBody()->getContents(),true);
        // Loop through each keyword and save to database
        foreach ($data['keywords'] as $keyword) {
            CampaignKeyword::updateOrCreate(
                ['keyword_id' => $keyword['keywordId']], // Prevent duplicates
                [
                    'campaign_id' => $keyword['campaignId'],
                    'keyword_id' => $keyword['keywordId'],
                    'keyword_text' => $keyword['keywordText'],
                    'ad_group_id' => $keyword['adGroupId'],
                    'match_type' => $keyword['matchType'],
                    'keyword_status' => $keyword['state'],
                    'normalized_keyword_text' => strtolower($keyword['keywordText']), // Example normalization
                    'bid_value' => $keyword['bid'],
                    'bid_currency' => 'USD', // Update if API returns currency
                ]
            );
        }
    }
    public function getCampaignAdsGroupList($accessToken,$profile_Id,$campaignId){
        // dd($accessToken,$profile_Id,$campaignId);
        // Define the API endpoint and request headers
        $url = $this->baseUrl.'/sp/adGroups/list';
        $response = $this->client->post($url, [
            'headers' => [
                'Authorization' => 'Bearer ' . $accessToken,
                'Amazon-Advertising-API-ClientId' => env('AMAZON_CLIENT_ID'),
                'Amazon-Advertising-API-Scope' => $profile_Id,
                'Content-Type' => 'application/vnd.spAdGroup.v3+json',
                'Accept' => 'application/vnd.spAdGroup.v3+json',
            ],
            'json' => [
                'campaignIdFilter' => [
                    'include' => [$campaignId]
                ],
                'stateFilter' => [
                    'include' => ['ENABLED', 'PAUSED', 'ARCHIVED']
                ],
                'maxResults' => 100,
                'locale' => 'en_US'
            ]
        ]);
        return json_decode($response->getBody()->getContents(),true);
        
    }
    public function generateAndDownloadReport($accessToken, $profile_Id, $campaignId)
    {
        $url = $this->baseUrl . '/reporting/reports';
        
        $startDate = date('Y-02-06');
        $endDate = date('Y-02-06');
        
        $data = [
            'name' => 'Campaign Report',
            'startDate' => $startDate,
            'endDate' => $endDate,
            'configuration' => [
                'adProduct' => 'SPONSORED_PRODUCTS',
                'groupBy' => ['campaign'],
                'columns' => ['campaignId', 'impressions', 'clicks', 'cost', 'attributedSalesSameSku14d', 'attributedSalesSameSku7d', 'attributedSalesSameSku1d', 'attributedSalesSameSku30d', 'sales1d', 'sales7d', 'sales14d', 'sales30d'],
                'reportTypeId' => 'spCampaigns',
                'timeUnit' => 'SUMMARY',
                "format" => "GZIP_JSON"
            ]
        ];
        
        Log::info('Sending report request', ['data' => $data]);
        
        $response = $this->client->post($url, [
            'headers' => [
                'Authorization' => 'Bearer ' . $accessToken,
                'Amazon-Advertising-API-ClientId' => env('AMAZON_CLIENT_ID'),
                'Amazon-Advertising-API-Scope' => $profile_Id,
                'Content-Type' => 'application/vnd.createasyncreportrequest.v3+json',
                'Accept' => 'application/vnd.createasyncreportrequest.v3+json'
            ],
            'json' => $data
        ]);
        
        $responseBody = json_decode($response->getBody()->getContents(), true);
        Log::info('Report request response', ['response' => $responseBody]);
        
        if (!isset($responseBody['reportId'])) {
            Log::error('Failed to generate report', ['response' => $responseBody]);
            return response()->json(['error' => 'Failed to generate report'], 500);
        }
        
        $reportId = $responseBody['reportId'];
        
        $statusUrl = $this->baseUrl . "/reporting/reports/{$reportId}";
        do {
            sleep(5);
            Log::info("Checking report status for Report ID: {$reportId}");
            
            $statusResponse = $this->client->get($statusUrl, [
                'headers' => [
                    'Authorization' => 'Bearer ' . $accessToken,
                    'Amazon-Advertising-API-ClientId' => env('AMAZON_CLIENT_ID'),
                    'Amazon-Advertising-API-Scope' => $profile_Id,
                    'Content-Type' => 'application/vnd.createasyncreportrequest.v3+json',
                    'Accept' => 'application/vnd.createasyncreportrequest.v3+json'
                ]
            ]);
            $statusData = json_decode($statusResponse->getBody()->getContents(), true);
            Log::info('Report status response', ['statusData' => $statusData]);
        } while (!isset($statusData['status']) || $statusData['status'] !== 'COMPLETED');
        
        if (!isset($statusData['url'])) {
            Log::error('Report URL not found', ['statusData' => $statusData]);
            return response()->json(['error' => 'Report URL not found'], 500);
        }
        
        $reportUrl = $statusData['url'];
        $reportResponse = $this->client->get($reportUrl);
        
        $filePath = storage_path("app/reports/campaign_report_{$reportId}.gz");
        file_put_contents($filePath, $reportResponse->getBody());
        
        Log::info('Report downloaded successfully', ['filePath' => $filePath]);
        
        return response()->json(['message' => 'Report downloaded successfully', 'path' => $filePath]);
    }







    public function getCampaignPerformanceData($accessToken,$profile_Id,$campaignId=null)
    {
        $url = $this->baseUrl.'/reporting/reports';

        $startDate = date('Y-m-d',strtotime('-1 day')); // First day of the current month
        $endDate = date('Y-m-d',strtotime('-1 day')); // Last day of the current month

        $data=[
            'name' => 'Campaign Report',
                'startDate' => $startDate, // January 2025
                'endDate' => $endDate,
                'configuration' => [
                    'adProduct' => 'SPONSORED_PRODUCTS',
                    'groupBy' => ['campaign'],
                    'columns' => ['campaignId', 'impressions', 'clicks', 'cost', 'attributedSalesSameSku14d','attributedSalesSameSku7d','attributedSalesSameSku1d','attributedSalesSameSku30d','sales1d','sales7d','sales14d','sales30d'],
                    'reportTypeId' => 'spCampaigns',
                    'timeUnit' => 'SUMMARY',
                    "format" => "GZIP_JSON"
                ]

        ];
        // dd($data);

        // Send the report creation request
        $response = $this->client->post($url, [
            'headers' => [
                'Authorization' => 'Bearer ' . $accessToken,
                'Amazon-Advertising-API-ClientId' => env('AMAZON_CLIENT_ID'),
                'Amazon-Advertising-API-Scope' => $profile_Id,
                'Content-Type' => 'application/vnd.createasyncreportrequest.v3+json',
                'Accept' => 'application/vnd.createasyncreportrequest.v3+json'
            ],
            'json' => $data
        ]);

        $responseBody = json_decode($response->getBody()->getContents(), true);
        return $responseBody;
    }
    public function checkReportStatus($accessToken,$profile_Id,$reportId)
    {
        $startDate = date('Y-m-d',strtotime('-1 day')); // First day of the current month
        $endDate = date('Y-m-d',strtotime('-1 day')); // Last day of the current month
        // Base URL for the reporting API
        $apiUrl = $this->baseUrl."/reporting/reports/{$reportId}";


        // try {
            // Make the GET request to check the status of the report
            $response = $this->client->get($apiUrl, [
                'headers' => [
                    'Authorization' => 'Bearer ' . $accessToken,
                    'Amazon-Advertising-API-ClientId' => env('AMAZON_CLIENT_ID'),
                    'Amazon-Advertising-API-Scope' => $profile_Id,
                    'Content-Type' => 'application/vnd.createasyncreportrequest.v3+json',
                    'Accept' => 'application/vnd.createasyncreportrequest.v3+json',
                ]
            ]);

            // Get the response body
            $responseBody = $response->getBody()->getContents();

            // Decode the response into an array
            $responseData = json_decode($responseBody, true);

            // Dump the response data
            // You can use dd() or var_dump() to view the output
            if($responseData['status']=='COMPLETED'){
                $reportUrl = $responseData['url'];
                $reportResponse = $this->client->get($reportUrl);
                
                // Define the file paths
                $gzFilePath = storage_path("app/reports/campaign_report_{$reportId}.gz");
                $jsonFilePath = storage_path("app/reports/campaign_report_{$reportId}.json");

                // Save the downloaded GZIP file
                file_put_contents($gzFilePath, $reportResponse->getBody());

                // Extract the JSON file from GZIP
                $gz = gzopen($gzFilePath, 'rb');
                $jsonData = '';
                while (!gzeof($gz)) {
                    $jsonData .= gzread($gz, 4096);
                }
                gzclose($gz);

                // Save the extracted JSON data to a file
                file_put_contents($jsonFilePath, $jsonData);

                // Process the JSON file
                if (file_exists($jsonFilePath)) {
                    $json = file_get_contents($jsonFilePath);
                    $data = json_decode($json, true);

                    if (!empty($data)) {
                        foreach ($data as $record) {
                            CampaignPerformanceMetric::updateOrCreate(
                                [
                                    'campaign_id' => $record['campaignId'],
                                    'start_date' => $startDate,
                                    'end_date' => $endDate
                                ],
                                [
                                    'impressions' => $record['impressions'] ?? 0,
                                    'clicks' => $record['clicks'] ?? 0,
                                    'cost' => $record['cost'] ?? 0,
                                    'sales1d' => $record['sales1d'] ?? 0,
                                    'sales7d' => $record['sales7d'] ?? 0,
                                    'sales14d' => $record['sales14d'] ?? 0,
                                    'sales30d' => $record['sales30d'] ?? 0,
                                    'attributed_sales_same_sku30d' => $record['attributedSalesSameSku30d'] ?? 0,
                                    'attributed_sales_same_sku14d' => $record['attributedSalesSameSku14d'] ?? 0,
                                    'attributed_sales_same_sku7d' => $record['attributedSalesSameSku7d'] ?? 0,
                                    'attributed_sales_same_sku1d' => $record['attributedSalesSameSku1d'] ?? 0
                                ]
                            );
                        }

                        // Delete the processed files
                        unlink($gzFilePath);
                        unlink($jsonFilePath);

                        Log::info('Report processed and files deleted successfully', [
                            'reportId' => $reportId,
                            'processedRecords' => count($data)
                        ]);

                        return [
                            'success' => true,
                            'message' => 'Report processed successfully',
                            'processed_records' => count($data)
                        ];
                    }
                }

                return [
                    'success' => false,
                    'message' => 'Failed to process report'
                ];
            }
            return [
                'success' => false,
                'message' => 'Failed to process report'
            ];

        // } catch (\Exception $e) {
        //     // Handle any exceptions (e.g., connection issues, bad responses)
        //     return response()->json([
        //         'error' => 'Error fetching report status: ' . $e->getMessage()
        //     ], 500);
        // }
    }
}